# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['register', 'regsyn', 'globsyn']

# %% ../nbs/00_core.ipynb 4
from typing import Optional, Callable, Set, Dict

# %% ../nbs/00_core.ipynb 7
def register(registry: Dict[str, Callable], fname: Optional[str] = None) -> Callable:
    '''Register an alias for a function
    
    Parameters
    ----------
    fname : Optional[str], default: `func.__name__`
        The name of the callable to register. Defaults to the decorated function's name.

    Returns
    -------
    Callable
        The decorated function
    '''
    def decorator(func: Callable) -> Callable:
        nonlocal registry, fname
        fname = fname or func.__name__
        registry |= {fname: func}
        return func
    return decorator

# %% ../nbs/00_core.ipynb 9
def regsyn(registry: Dict[str, Callable], synset: Optional[Set[str]] = None) -> Callable:
    '''Register a set of synonyms for a function
    
    Parameters
    ----------
    synset : Optional[Set[str]]
        The synonyms of the function to register.

    Returns
    -------
    Callable
        The function to be decorated
    '''    
    def decorator(func: Callable) -> Callable:
        nonlocal registry, synset
        synset = synset or {func.__name__}
        for syn in synset:
            register(registry, syn)(func)        
        return func
    return decorator


# %% ../nbs/00_core.ipynb 11
def globsyn(synset: Set[str], globs: Optional[Dict[str, Callable]] = None) -> Callable:
    '''Register a set of synonyms for a function to globals()
    
    Parameters
    ----------
    synset : Set[str]
        The synonyms of the function to register.

    Returns
    -------
    Callable
        The decorated function
    '''
    globs = globs or globals()
    def decorator(func: Callable) -> Callable:   
        regsyn(registry=globs, synset=synset)(func)        
        return func
    return decorator

